// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
// 
// This program is free software; you can redistribute it and/or 
// modify it under the terms of the GNU General Public License 
// as published by the Free Software Foundation; either 
// version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#ifndef GLOBAL_H
#define GLOBAL_H
#include "singleton.h"
#include "utils.h"

class Global: public Singleton<Global> {
public:
	bool addallmaster;
	bool addalluser;
	bool dumpall;
	bool fixeddict;
	int xsize;
	int ysize;

public:
	// global defaults
	Global(): addallmaster(false), addalluser(false), dumpall(false), fixeddict(false),
	          xsize(400), ysize(400) {}
	
     void getCfg() {
		LOG("Global::getCfg");
		char *fname = getCfgFname();
		char *cfg = UTILS::read_file_malloc(fname);
		if (cfg == 0) {
			delete[] fname;
			return;
		}
		
		if (strstr(cfg, "fixed_dict") != 0)
			fixeddict = true;
		
		char *ptr;
		if ((ptr = strstr(cfg, "xsize")) != 0) {
			int n = getNumber(ptr);
			if (n > 0)
				xsize = n;
		}
		if ((ptr = strstr(cfg, "ysize")) != 0) {
			int n = getNumber(ptr);
			if (n > 0)
				ysize = n;
		}
		
		delete[] fname;
		delete[] cfg;
	}
	
     void saveCfg() {
		char *fname = getCfgFname();
		char *cfg = new char[1000];
		memset(cfg, 0, 1000);
		strcpy(cfg, "rospell configuration\n");
		
		if (fixeddict == true)
			strcat(cfg, "fixed_dict\n");
		if (xsize > 0) {
			char tmp[100];
			sprintf(tmp, "xsize %d\n", xsize);
			strcat(cfg, tmp);
		}
		if (ysize > 0) {
			char tmp[100];
			sprintf(tmp, "ysize %d\n", ysize);
			strcat(cfg, tmp);
		}
		
		UTILS::write_file(fname, cfg);
		
		delete[] fname;
		delete[] cfg;
	}
	
	
private:
	int getNumber(char *ptr) {
		
		ASSERT(ptr != 0);
		while (*ptr != ' ') {
			if (*ptr == '\0')
				return 0;
			ptr++;
		}
		
		int retval;
		sscanf(ptr, "%d", &retval);
		LOG("Global::getNumber %d", retval);
		return retval;
	}

	char *getCfgFname() {
		char *homedir = getenv("HOME");
		if (homedir == 0) {
			fprintf(stderr, "Error: HOME environment variable nu a fost găsită\n");
			exit(1);
		}
		
		unsigned len = strlen(homedir) + strlen("/.rospell/config");
		char *fname = new char[len + 1];
		memset(fname, '\0', len + 1);
		strcpy(fname, homedir);
		strcat(fname, "/.rospell/config");
		return fname;
	}	
	
};

#endif

