// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
// 
// This program is free software; you can redistribute it and/or 
// modify it under the terms of the GNU General Public License 
// as published by the Free Software Foundation; either 
// version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///****************************************************************************
//** $Id: qt/qwerty.cpp   3.3.4   edited May 27 2003 $
//**
//** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
//**
//** This file is part of an example program for Qt.  This example
//** program may be used, distributed and modified without limitation.
//**
//*****************************************************************************/

#include "qwerty.h"
#include <qapplication.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qpopupmenu.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qpaintdevicemetrics.h>
#include <qptrlist.h>
#include <qfontdialog.h>
//#include <qstatusbar.h>

#include <qtextcodec.h>
#include "spell.h"
#include "global.h"
#include "utils.h"
#include "stats.h"

const bool no_writing = FALSE;


SyntaxHighlighter::SyntaxHighlighter(QTextEdit *textEdit) 
  : QSyntaxHighlighter(textEdit) {
  return;
}

SyntaxHighlighter::~SyntaxHighlighter(void) {
  return;
}

int SyntaxHighlighter::highlightParagraph(const QString &text, 
					  int endStateOfLastPara) {

     int x = 0;
	QChar ch = text.at(x);
	int start = (endStateOfLastPara == 1)? 0:-1;
	while (!ch.isNull()) {
		if (ch == '<')
			start = x;
		if (ch == '>' && start != -1) {
			setFormat(start, x - start + 1, QColor(255, 25, 25));
			start = -1;
		}
		ch = text.at(++x);
	}
	
	if (start != -1) {
		setFormat(start, x - start, QColor(255, 0, 0));
		return 1;
	}
	return 0;
}



Editor::Editor( QWidget * parent , const char * name )
    : QWidget( parent, name, WDestructiveClose )
{

	pdict_ = new Dictionaries(true);	
	readHomeDir(pdict_);
	
	search_ = "";
	format_ = FILE_UTF8;
	*word1_ = '\0';
	*word2_ = '\0';
	*word3_ = '\0';
	*word4_ = '\0';
	*word5_ = '\0';
	

	for (int i = 0; i < MAXBOOKMARKS; i++) {
		xbook_[i] = 0;
		ybook_[i] = 0;
	}
	
	QAccel *a_ = new QAccel(this);        // create accels for myWindow
	a_->connectItem(a_->insertItem(ALT+Key_A), this, SLOT(unicode_a()));
	a_->connectItem(a_->insertItem(SHIFT+ALT+Key_A), this, SLOT(unicode_A()));
	a_->connectItem(a_->insertItem(ALT+Key_S), this, SLOT(unicode_s()));
	a_->connectItem(a_->insertItem(SHIFT+ALT+Key_S), this, SLOT(unicode_S()));
	a_->connectItem(a_->insertItem(ALT+Key_Q), this, SLOT(unicode_q()));
	a_->connectItem(a_->insertItem(SHIFT+ALT+Key_Q), this, SLOT(unicode_Q()));
	a_->connectItem(a_->insertItem(ALT+Key_T), this, SLOT(unicode_t()));
	a_->connectItem(a_->insertItem(SHIFT+ALT+Key_T), this, SLOT(unicode_T()));
	a_->connectItem(a_->insertItem(ALT+Key_I), this, SLOT(unicode_i()));
	a_->connectItem(a_->insertItem(SHIFT+ALT+Key_I), this, SLOT(unicode_I()));
	
    m_ = new QMenuBar( this, "menu" );



    QPopupMenu * file_ = new QPopupMenu(m_);
    Q_CHECK_PTR( file_ );
    m_->insertItem( "&File", file_ );

    file_->insertItem( "&New",   this, SLOT(newDoc()), CTRL+Key_N );
    file_->insertItem( "&Open...",  this, SLOT(load()), CTRL+Key_O );
    file_->insertItem( "Open &Location",  this, SLOT(openWwwLocation()), CTRL+Key_L);
    file_->insertItem( "&Save...",  this, SLOT(save()), CTRL+Key_S );
    file_->insertItem( "Sa&ve As",  this, SLOT(saveAs()), SHIFT+CTRL+Key_S);
    file_->insertItem( "E&xit", this, SLOT(close()));

    file_->insertSeparator();
#ifndef QT_NO_PRINTER
    file_->insertItem( "&Print...", this, SLOT(print()), CTRL+Key_P );
#endif


    QPopupMenu *edit_ = new QPopupMenu(m_);
    Q_CHECK_PTR(edit_);
    m_->insertItem("&Edit", edit_);

    edit_->insertItem( "&Undo", this, SLOT(undo()), CTRL+Key_Z);
    edit_->insertSeparator();
    edit_->insertItem( "Cut", this, SLOT(cut()), CTRL+Key_X);
    edit_->insertItem( "Copy", this, SLOT(copy()), CTRL+Key_C);
    edit_->insertItem( "Paste", this, SLOT(paste()), CTRL+Key_V);
    edit_->insertItem( "Delete", this, SLOT(e_delete()));
    edit_->insertSeparator();
    edit_->insertItem( "&Find", this, SLOT(search()), CTRL+Key_F);
    edit_->insertItem( "Find Ne&xt", this, SLOT(searchAgain()), CTRL+Key_G);
    edit_->insertItem( "Goto &Line", this, SLOT(gotoLine()), CTRL+Key_I);
    edit_->insertItem( "Set &Bookmark", this, SLOT(setBookmark()));
    edit_->insertItem( "Goto Boo&kmark", this, SLOT(gotoBookmark()));


    QPopupMenu * tools_ = new QPopupMenu(m_);
    Q_CHECK_PTR( tools_ );
    m_->insertItem( "T&ools", tools_ );

    tools_->insertItem( "Select &Font" ,   this, SLOT(font()));
    tools_->insertItem( "S&tatus" ,   this, SLOT(status()));
    tools_->insertSeparator();
    tools_->insertItem( "&Uppercase" ,   this, SLOT(uppercase()));
    tools_->insertItem( "&Lowercase" ,   this, SLOT(lowercase()));
    tools_->insertItem( "&dexonline.ro" ,   this, SLOT(dexonline()));



    m_->insertItem( "    ",   this, 0, 0 );
    msgid_ = m_->insertItem( "         ",   this, 0, 0 );




	// words spelling menubar	
	QVBoxLayout *vly = new QVBoxLayout(this);
	vly->setAutoAdd(true);
	QVBox *vbox = new QVBox(this);
	mwords_ = new QMenuBar( vbox, "words menu" );

    mwords_->insertItem( "Sta&rt Spelling",   this, SLOT(spellAdvance()),   ALT+Key_R );
    mwords_->insertItem( "I&gnore",   this, SLOT(spellIgnore()),   ALT+Key_G );
    mwords_->insertItem( "A&dd",   this, SLOT(spellAdd()),   ALT+Key_D );
	if (Global::instance()->fixeddict)
	    mwords_->insertItem( "Add na&me",   this, SLOT(spellName()),   ALT+Key_M );
    mw1_ = mwords_->insertItem( " ",   this, SLOT(replaceWord1()), 0 );
    mw2_ = mwords_->insertItem( " ",   this, SLOT(replaceWord2()), 0 );
    mw3_ = mwords_->insertItem( " ",   this, SLOT(replaceWord3()), 0 );
    mw4_ = mwords_->insertItem( " ",   this, SLOT(replaceWord4()), 0 );
    mw5_ = mwords_->insertItem( " ",   this, SLOT(replaceWord5()), 0 );

    changed_ = FALSE;
    e_ = new QMultiLineEdit(vbox, "editor" );
    syntax_ = new SyntaxHighlighter(e_);
    connect( e_, SIGNAL( textChanged() ), this, SLOT( textChanged() ) );
    connect( e_, SIGNAL( clicked() ), this, SLOT( clicked() ) );

    //    QFont courier("Courier New",16,50); courier.setPixelSize(16);
    //    e_->setFont(courier);

	e_->setWordWrap(QTextEdit::NoWrap);

    e_->setFocus();


//QMessageBox::warning(0,"Charmap error", "some text here");

//	new QStatusBar(this, "status bar");
}

Editor::~Editor()
{
	LOG("Editor::~Editor");
	delete syntax_;
	delete pdict_;
	delete m_;
	delete mwords_;
	delete e_;
	
//#ifndef QT_NO_PRINTER lucian: will not compile when in
//	delete printer_;
//#endif
}


void Editor::dexonline() {
	char *wget = UTILS::run_prg_malloc("which wget");
	if (strstr(wget, "wget") == 0) {
		QMessageBox::warning(0,"Error", "wget not installed on your system");
		delete[] wget;
		return;
	}
	char *ptr = strstr(wget, "\n");
	if (ptr != 0)
		*ptr = '\0';
	
	QString sword = e_->markedText();
	
	if (sword.isNull()) {
		LOG("dexonline no word");
		return;
	}
	QCString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::dexonline  %s", cstr);
	msgScreen("Loading...");
	
	char cmd[1000];
	snprintf(cmd, 1000, "%s -q --output-document=- http://dexonline.ro/search.php?cuv=%s&source=", wget, cstr);
	char *www_content = UTILS::run_prg_malloc(cmd);
	msgScreen(0);
	if (www_content != 0) {
//		FILE_FORMAT format = detectWwwFormat(www_content);
//		QString txt = toUtf8(www_content, format);
		char *newcontent = stripDexonline(www_content);
		if (newcontent == 0) {
			delete[] www_content;
			return;
		}
		LOG("******\n%s\n***************\n", newcontent);
//		QMessageBox::warning(0, cstr, txt);
		QMessageBox::warning(0, "http://dexonline.ro", QString::fromUtf8(newcontent));
		delete[] newcontent;
	}
}


char *Editor::stripDexonline(char *str) {
	ASSERT(str != 0);
	if (str == 0)
		return 0;
		
	// strip <script>
	char *ptr1 = str;
	char *ptr2 = str;
	int cnt = 0;
	bool more = false;
	while ((ptr1 = strcasestr(ptr1, "<p><b>")) != 0) {
		if (cnt == 3) {
			more = true;
			break;
		}
		while ((unsigned) ptr2 != (unsigned) ptr1) {
			*ptr2 = ' ';
			ptr2++;
		}
		ptr1 = strcasestr(ptr1, "<table");
		if (ptr1 == 0)
			return 0;
		ptr2 = ptr1;
		cnt++;
	}
	
	*ptr2 = '\0';
	if (more)
		strcat(str, "<br><br>more...");
	
	ptr1 = UTILS::buffer_sed(str, "&#x103;", "ă");
	delete[] str;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xee;", "î");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xe2;", "â");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#x2013;", "-");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xcd;", "I");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#x102;", "Ă");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xc2;", "Â");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xc9;", "E");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xce;", "Î");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xd3;", "O");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xda;", "U");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xc1;", "A");
	delete[] ptr1;
		
	return ptr2;
//	return ptr1;
}
		
	




void Editor::uppercase() {
	QString text = e_->markedText();
	QCString utext = text.utf8();
	const char *ctext = utext;
	char *word = UTF8::strtoupper(ctext);
     e_->cut(); 
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	delete word;
}

void Editor::lowercase() {
	QString text = e_->markedText();
	QCString utext = text.utf8();
	const char *ctext = utext;
	char *word = UTF8::strtolower(ctext);
     e_->cut(); 
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	delete word;
}

char *Editor::format2str() {
	char *sformat = "unknown";
	switch (format_) {
		case FILE_UTF8:
			sformat = "Unicode UTF-8";
			break;
			
		case FILE_ISO8859_2:
			sformat = "ISO8859-2 (Latin 2)";
			break;
		 
		case FILE_ISO8859_16:
			sformat = "ISO8859-16 (South-Eastern European)";
			break;
		 
		case FILE_CP1250:
			sformat = "CP1250 (Microsoft Latin II)";
			break;
		
		default:
			ASSERT(0);
			break;
	}
	return sformat;
}

void Editor::status() {
	char str[1000];
	
	QString fname = caption();
	QCString ufname = fname.utf8();
	const char *csname = ufname;
	
	char *sformat = format2str();
	
	int lines = e_->numLines();
	
	char speller_stats[100];
	Stats::instance()->sprint(speller_stats, 100);
	
	QFont fnt = e_->font();
	const char *ffamily = (const char *) fnt.family().ascii();
	
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
		
	snprintf(str, 1000, "\n    rospell version %s    \n\n    File: %s\n    Loaded as: %s\n"
		"    Lines: %d\n\n"
		"    Font family: %s    \n"
		"    Font test: ăşâţîĂŞÂŢÎ    \n"
		"\n%s\n"
		"    Cursor: %d/%d\n",
		VERSION, csname, sformat, lines, ffamily, speller_stats, para + 1, index + 1);
		
	StatusDialog *sd = new StatusDialog(fnt, str);
	sd->setCaption("File status");
	sd->exec();
//	QMessageBox::warning(0,"File status", str);
	delete sd;
}

void Editor::font()
{
    bool ok;
    QFont f = QFontDialog::getFont( &ok, e_->font() );
    if ( ok ) {
        e_->setFont( f );
	   LOG("set font %s", (const char *) (f.family()).utf8());
	   mwords_->setFont(f);
    }
}

void Editor::newDoc()
{
    LOG("newDoc");
    Editor *ed = new Editor;
    if ( qApp->desktop()->size().width() < 450
         || qApp->desktop()->size().height() < 450 ) {
        ed->showMaximized();
    } else {
        ed->resize(Global::instance()->xsize, Global::instance()->ysize);
        LOG("Editor::newDoc sizes %d %d", Global::instance()->xsize, Global::instance()->ysize);
        ed->show();
    }
}

Editor::FILE_FORMAT Editor::detectWwwFormat(const char *str) {
	ASSERT(str != 0);
	FILE_FORMAT retval = FILE_UTF8;
	
	// detect iso8859-2
	while (1) {
		char *ptr = strcasestr(str, "<head>");
		if (ptr == 0)
			break;
		ptr = strcasestr(ptr, "charset=iso-8859-2");
		if (ptr == 0)
			break;
		ptr = strcasestr(ptr, "</head>");
		if (ptr == 0)
			break;
		retval = FILE_ISO8859_2;
		break;
	}
	
	// detect win1250
	while (1) {
		char *ptr = strcasestr(str, "<head>");
		if (ptr == 0)
			break;
		ptr = strcasestr(ptr, "charset=windows-1250");
		if (ptr == 0)
			break;
		ptr = strcasestr(ptr, "</head>");
		if (ptr == 0)
			break;
		retval = FILE_CP1250;
		break;
	}
	
	return retval;
}

QString Editor::toUtf8(const char *str, FILE_FORMAT format) {
	QString retval;
	
	switch (format) {
		case FILE_UTF8: {
			retval = QString::fromUtf8(str);
			LOG("Editor::toUtf8 utf8");
			}
			break;
		case FILE_ISO8859_2: {                    
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-2");
				QTextDecoder *decoder = codec->makeDecoder();
				retval = decoder->toUnicode(str, strlen(str));
				LOG("Editor::toUtf8 iso8859-2");
			}
			break;
		case FILE_ISO8859_16: {                    
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-16");
				QTextDecoder *decoder = codec->makeDecoder();
				retval = decoder->toUnicode(str, strlen(str));
				LOG("Editor::toUtf8 iso8859-16");
			}
			break;
		case FILE_CP1250: {
				QTextCodec *codec = QTextCodec::codecForName("CP1250");
				QTextDecoder *decoder = codec->makeDecoder();
				retval = decoder->toUnicode(str, strlen(str));
				LOG("Editor::toUtf8 cp1250");
			}
			break;
		default:
			ASSERT(0);
	}
	
	return retval;
}

void Editor::openWwwLocation()
{
	char *wget = UTILS::run_prg_malloc("which wget");
	if (strstr(wget, "wget") == 0) {
		QMessageBox::warning(0,"Error", "wget not installed on your system");
		delete[] wget;
		return;
	}
	char *ptr = strstr(wget, "\n");
	if (ptr != 0)
		*ptr = '\0';
	
	WwwDialog *wd = new WwwDialog();
	wd->setCaption("Enter document location");
	wd->exec();
	QString sword = wd->getWord();
	
	if (sword == "")
		return;
	QCString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::openWwwLocation  %s", cstr);
	delete wd;
	msgScreen("Loading...");
	
	char cmd[1000];
	snprintf(cmd, 1000, "%s -q --output-document=- %s", wget, cstr);
	char *www_content = UTILS::run_prg_malloc(cmd);
	if (www_content != 0) {
		FILE_FORMAT format = detectWwwFormat(www_content);
		QString txt = toUtf8(www_content, format);
		e_->setText(txt);
		sword.replace(QChar('/'), "_");
		setCaption(sword);
		format_ = format;
		msgScreen(format2str(), 2000);
		msgScreen(0);
//		QMessageBox::warning(0,"cstr", txt);
		delete[] www_content;
	}
}

void Editor::msgScreen(char *str, int mstime) {
	ASSERT(time >= 0);
	if (str == 0)
		m_->changeItem(msgid_, QString("          "));
	else
		m_->changeItem(msgid_, QString(str));
		
	LOG("mstime %d", mstime);
	if (mstime == 0) {
		qApp->processEvents();
		return;
	}
	
	QTime t;
	t.start();
	while( t.elapsed() < mstime) {
		qApp->processEvents();
	}	
}



void Editor::setFormat(FILE_FORMAT format) {
	format_ = format;
}

void Editor::load()
{
    QString fn = QFileDialog::getOpenFileName( QString::null, QString::null, this );
    if ( !fn.isEmpty() )
        load(fn, format_);
}

void Editor::load( const QString& fileName, FILE_FORMAT format)
{
	format_ = format;
	QCString qfname = fileName.utf8();
	const char *fname = qfname;
	LOG("Editor::load %s %d", fname, (int) format);

	e_->setAutoUpdate( FALSE );
    
	char *content = UTILS::read_file_malloc(fname);
	if (content == 0)
		return;
		
	QString txt = toUtf8(content, format);
	e_->setText(txt);
	delete[] content;

	e_->setAutoUpdate( TRUE );
	e_->repaint();
	setCaption( fileName );
	format_ = format;

    changed_ = FALSE;
	cleanMWords();
}

bool Editor::save()
{
	QString fname = caption();
	if (fname.isNull())
		return saveAs();
	else {
		QCString utext = fname.utf8();
		const char *cstr = utext;
		LOG("Editor::save file %s", cstr);
		if ( !fname.isEmpty() )
     		return saveCommon(fname);
		return FALSE;
	}
}

bool Editor::saveAs()
{
    //storing filename (proper save) is left as an exercise...
    QString fn = QFileDialog::getSaveFileName( QString::null, QString::null, this );
    if ( !fn.isEmpty() )
        return saveCommon(fn);
    return FALSE;
}



bool Editor::saveCommon(const QString& fileName)
{
    QFile f( fileName );
    if ( no_writing || !f.open( IO_WriteOnly ) ) {
        QMessageBox::warning(this,"I/O Error",
                    QString("The file could not be opened.\n\n")
                        +fileName);
        return FALSE;
    }
    QTextStream t(&f);
    LOG("save common format %d", format_);

		switch (format_) {
			case FILE_UTF8:
				t.setEncoding( QTextStream::UnicodeUTF8 );
				LOG("Editor::saveCommon utf8");
				break;
			case FILE_CP1250: {
				// lucian: qt has some problems with this conversion!!!                    
				QTextCodec *codec = QTextCodec::codecForName("CP1250");
				t.setCodec(codec); 
				LOG("Editor::saveCommon cp1250");
				}
				break;
			case FILE_ISO8859_2: {
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-2");
				t.setCodec(codec); 
				LOG("Editor::saveCommon iso8859-2");
				}
				break;
			case FILE_ISO8859_16: {
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-16");
				t.setCodec(codec); 
				LOG("Editor::saveCommon iso8859-16");
				}
				break;
			default:
				ASSERT(0);
				return FALSE;
    	}
	
    t << e_->text();
    f.close();
    setCaption( fileName );
    changed_ = FALSE;
	cleanMWords();
    return TRUE;
}

void Editor::print()
{
#ifndef QT_NO_PRINTER
    if ( printer_.setup(this) ) {                // opens printer dialog
        printer_.setFullPage(TRUE);              // we'll set our own margins
        QPainter p;
        p.begin( &printer_ );                    // paint on printer
        p.setFont( e_->font() );
        QFontMetrics fm = p.fontMetrics();
        QPaintDeviceMetrics metrics( &printer_ ); // need width/height
                                                 // of printer surface
        const int MARGIN = metrics.logicalDpiX() / 2; // half-inch margin
        int yPos        = MARGIN;               // y position for each line

        for( int i = 0 ; i < e_->numLines() ; i++ ) {
            if ( printer_.aborted() )
                break;
            if ( yPos + fm.lineSpacing() > metrics.height() - MARGIN ) {
                // no more room on this page
                if ( !printer_.newPage() )          // start new page
                    break;                           // some error
                yPos = MARGIN;                   // back to top of page
            }
            p.drawText( MARGIN, yPos, metrics.width() - 2*MARGIN,
                        fm.lineSpacing(), ExpandTabs, e_->textLine( i ) );
            yPos += fm.lineSpacing();
        }
        p.end();                                // send job to printer
    }
	cleanMWords();
#endif
}

void Editor::resizeEvent( QResizeEvent * )
{
LOG("resizeEvent sizes %d %d", width(), height());
    if ( e_ && m_ )
        e_->setGeometry( 0, m_->height(), width(), height() - m_->height() );
    Global::instance()->xsize = width();
    Global::instance()->ysize = height();
}

void Editor::closeEvent( QCloseEvent *event )
{
LOG("closeEvent");
    event->accept();

    if ( changed_ ) { // the text has been changed
        switch ( QMessageBox::warning( this, "Question",
                                        "Save changes to Document?",
                                        tr("&Yes"),
                                        tr("&No"),
                                        tr("Cancel"),
                                        0, 2) ) {
        case 0: // yes
            if ( save() )
                event->accept();
            else
                event->ignore();
            break;
        case 1: // no
            event->accept();
            break;
        default: // cancel
            event->ignore();
            break;
        }
    }
}



void Editor::advanceStartWord() {

	LOG("enter advanceStartWord function with index %d", x_);
	int lines = e_->numLines();
	LOG("lines %d", lines);
	
	while (1) {
		QChar ch = line_.at(x_);
		for (int i = 0; !ch.isNull(); i++, x_++) {
			ch = line_.at(x_);
			if (ch.isLetterOrNumber() || ch == '<' || ch == '/' || ch == '&') { // check for utf8 also
				LOG("return advanceStartWord function with index %d", x_);
				return;
			}
		}
		
		if ((y_ + 1) >= lines) {
			LOG("advanceStartWord advance end of file detected");
			e_->setCursorPosition(y_, x_, false);
			break;
		}
		
		y_++;
		x_ = 0;
		line_ = e_->textLine(y_);
		e_->setCursorPosition(y_, x_, false);
		LOG("advanceStartWord advance para %d, index %d", y_, x_);
	}
}

QString Editor::advanceEndWord() {
	if (e_->atEnd())
		return QString("");
	
	xword_ = x_;
	LOG("enter advanceEndWord with index %d", x_);
	
	QChar ch = line_.at(x_);
	int i;
	for (i = 0; !ch.isNull() && (ch.isLetterOrNumber() || ch =='-'); i++) {
		x_++;
		ch = line_.at(x_);
	}
	if (i == 0) {
		LOG("return from 0");
		return QString("");
	}
	
	
	LOG("advanceWord selection %d/%d %d/%d", y_, xword_, y_, x_);
	//lucian: it might return all string!!!
	QString retval = line_.mid(xword_, x_ - xword_);
	if (retval.isNull()) {
		LOG("return isNull");
		return QString("");
	}
		
//	LOG("advanceEndWord %s", (const char *)retval.utf8());
	LOG("exit advanceEndWord with index %d", x_);
	return retval;
}


bool Editor::skeepXML() {
	LOG("enter skeepXML with index %d", x_);
	int lines = e_->numLines();
	LOG("lines %d", lines);
	QChar ch = line_.at(x_);
	
	if (ch == '<') {
		LOG("Editor::skeepXML - xml detected");
		while (1) {
			x_++;
			ch = line_.at(x_);
			if (ch == '>')
				break;
			if (ch.isNull()) {
				// advance here to a new line
				if ((y_ + 1) > lines) {
					LOG("advanceXML end of file detected");
					e_->setCursorPosition(y_, x_, false);
					return true;
				}
				y_++;
				x_ = 0;
				line_ = e_->textLine(y_);
				LOG("skeepXML sml advance para %d, index %d", y_, x_);
			}
		}
		
		LOG("exit skeepXML with index %d", x_);
		return true;
	}
	else if (ch == '/' || ch == '&') {
		LOG("Editor::skeepXML - path detected");
		x_++;
		QChar ch = line_.at(x_);
		while (ch.isLetterOrNumber()) {
			x_++;
			ch = line_.at(x_);
		}
		LOG("exit skeepXML path with index %d", x_);
		return true;
	}
	
	
	LOG("exit skeepXML with index %d", x_);
	return false;

}

void Editor::spellAdvance()
{
	if (e_->atEnd()) {
		e_->selectAll(false);
		return;
	}
		
//	e_->moveCursor(QTextEdit::MoveForward, true);
//	e_->moveCursor(QTextEdit::MoveBackward, true);
		
	e_->getCursorPosition(&y_, &x_);
	line_ = e_->textLine(y_);
												
	// just in case the cursor is in the middle of a word		
	advanceEndWord();
	
	DbDict *hptr = pdict_->getMaster();
	DbUser *uptr = pdict_->getUser();
	DbUser *iptr = pdict_->getIgnore();
	DbUser *fptr = pdict_->getFixed();
	while (1) {
  	   e_->selectAll(false);
		// advance to a new word
		advanceStartWord();		
		if (e_->atEnd()) {
			cleanMWords();
			return;
		}
		
		// skeep XML markup
		if (skeepXML())
			continue;
		
		//e->cursorWordForward(true);
		QString text = advanceEndWord();
		
		if (text.isEmpty() || text.isNull())
			continue;
			
		// look for word in dictionary
		QCString utext = text.utf8();
		const char *ctext = utext; 
		
		LOG("text:%s", ctext);
		//lucian: to skeep <code> blocks
		//lucian: to skeep <mail, <date, <path
		
		// text with numbers is assumed not to be a target word for spelling
		// example: hda3
		if (hasnumeric(ctext))
			continue;
		
		char *active_word = extract_word((char *) ctext);
		if (active_word == 0) {
			// lucian to fix1: we might get here because the word started with an
			// utf8 char that is not a valid word char
			// just jump over this char for now
			e_->moveCursor(QTextEdit::MoveForward, false);
			continue;
		}
			
		LOG("text active:%s", active_word);
		if (!hptr->findString(active_word) &&   // if not in dictionary
		    !fptr->findString(active_word, true) &&   // if not in fixed/names dictionary
		    !uptr->findString(active_word) &&   // if not in user dictionary
		    !iptr->findString(active_word)) {   // if not in ignore dictionary
			Stats::instance()->notFound();
			cleanMWords();
			// skeep lookalike for words consisting in a single letter
			if (strlen(active_word) == 1) {
				delete[] active_word;
				break;
			}
			
			// grab any valid words form dict based on active_words
			hptr->findLookalike(active_word, word1_, word2_, word3_, word4_, word5_);
			LOG("find lookalike dictionary done");
     		mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
     		mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
     		mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
     		mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
     		mwords_->changeItem(mw5_, QString::fromUtf8(word5_));
		
			// look for words with i to â rule
			bool look_done = false;		
			if (*word1_ == '\0' && !look_done) {
				look_done = lookalike1(active_word, word1_);
				if (look_done)
		     		mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
			}
			if (*word2_ == '\0' && !look_done) {
				look_done = lookalike1(active_word, word2_);
				if (look_done)
		     		mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
			}
			if (*word3_ == '\0' && !look_done) {
				look_done = lookalike1(active_word, word3_);
				if (look_done)
		     		mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
			}
			if (*word4_ == '\0' && !look_done) {
				look_done = lookalike1(active_word, word4_);
				if (look_done)
		     		mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
			}
			if (*word5_ == '\0' && !look_done) {
				look_done = lookalike1(active_word, word5_);
				if (look_done)
		     		mwords_->changeItem(mw5_, QString::fromUtf8(word5_));
			}
			LOG("find lookalike1 done");
		     
			// look for the second apparition in words with i to â rule
			look_done = false;		
			if (*word1_ == '\0' && !look_done) {
				look_done = lookalike2(active_word, word1_);
				if (look_done)
		     		mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
			}
			if (*word2_ == '\0' && !look_done) {
				look_done = lookalike2(active_word, word2_);
				if (look_done)
		     		mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
			}
			if (*word3_ == '\0' && !look_done) {
				look_done = lookalike2(active_word, word3_);
				if (look_done)
		     		mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
			}
			if (*word4_ == '\0' && !look_done) {
				look_done = lookalike2(active_word, word4_);
				if (look_done)
		     		mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
			}
			if (*word5_ == '\0' && !look_done) {
				look_done = lookalike2(active_word, word5_);
				if (look_done)
		     		mwords_->changeItem(mw5_, QString::fromUtf8(word5_));
			}
			LOG("find lookalike2 done");
		
			// look for the first apparition in words with î to â rule
			look_done = false;		
			if (*word1_ == '\0' && !look_done) {
				look_done = lookalike3(active_word, word1_);
				if (look_done)
		     		mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
			}
			if (*word2_ == '\0' && !look_done) {
				look_done = lookalike3(active_word, word2_);
				if (look_done)
		     		mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
			}
			if (*word3_ == '\0' && !look_done) {
				look_done = lookalike3(active_word, word3_);
				if (look_done)
		     		mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
			}
			if (*word4_ == '\0' && !look_done) {
				look_done = lookalike3(active_word, word4_);
				if (look_done)
		     		mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
			}
			if (*word5_ == '\0' && !look_done) {
				look_done = lookalike3(active_word, word5_);
				if (look_done)
		     		mwords_->changeItem(mw5_, QString::fromUtf8(word5_));
			}
			LOG("find lookalike3 done");
		
			LOG("word1:%s", word1_);
			LOG("word2:%s", word2_);
			LOG("word3:%s", word3_);
			LOG("word4:%s", word4_);
			LOG("word5:%s", word5_);
			delete[] active_word;
			
			break;
		}
		else
			Stats::instance()->found();
		
		
		delete[] active_word;
	}
	
	e_->setCursorPosition(y_, x_, false);
	e_->setSelection(y_, xword_, y_, x_);
}

// first i to â lookalike
bool Editor::lookalike1(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "i")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "i");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "â");
		strcat(ptrtmp, ptr + 1);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}
		
// second i to â lookalike
bool Editor::lookalike2(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "i")) != 0) {
		if ((ptr = strstr(ptr + 1, "i")) != 0) {
			strcpy(dest, active_word);
			ptr = strstr(dest, "i");
			ASSERT(ptr != 0);
			if (ptr == 0)
				return false; // just in case
			ptr = strstr(ptr + 1, "i");
			ASSERT(ptr != 0);
			if (ptr == 0)
				return false; // just in case
			char ptrtmp[100];
			strcpy(ptrtmp, "â");
			strcat(ptrtmp, ptr + 1);
			strcpy(ptr, ptrtmp);
			// look for the new word in dictionary
			DbDict *hptr = pdict_->getMaster();
			if (hptr->findString(dest))
				return true;
			else
				*dest = '\0';
		}
	}
	return false;
}
		
// first î to â lookalike
bool Editor::lookalike3(char *active_word, char *dest) {
	ASSERT(active_word != 0);
	ASSERT(dest != 0);
	char *ptr;
	if ((ptr = strstr(active_word, "î")) != 0) {
		strcpy(dest, active_word);
		ptr = strstr(dest, "î");
		ASSERT(ptr != 0);
		if (ptr == 0)
			return false; // just in case
		char ptrtmp[100];
		strcpy(ptrtmp, "â");
		strcat(ptrtmp, ptr + 2);
		strcpy(ptr, ptrtmp);
		// look for the new word in dictionary
		DbDict *hptr = pdict_->getMaster();
		if (hptr->findString(dest))
			return true;
		else
			*dest = '\0';
	}
	return false;
}
		
void Editor::cleanMWords() {
	mwords_->changeItem(mw1_, QString(" "));
	mwords_->changeItem(mw2_, QString(" "));
	mwords_->changeItem(mw3_, QString(" "));
	mwords_->changeItem(mw4_, QString(" "));
	mwords_->changeItem(mw5_, QString(" "));
	*word1_ = '\0';
	*word2_ = '\0';
	*word3_ = '\0';
	*word4_ = '\0';
	*word5_ = '\0';
}

void Editor::replaceWord1() {
	if (*word1_ == ' ' || *word1_ == '\0')
		return;
LOG("replaceWord1 %s", word1_);
	char word[100];
	strcpy(word, word1_);
     e_->cut(); 
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord2() {
	if (*word2_ == ' ' || *word2_ == '\0')
		return;
LOG("replaceWord2 %s", word2_);
	char word[100];
	strcpy(word, word2_);
     e_->cut(); 
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord3() {
	if (*word3_ == ' ' || *word3_ == '\0')
		return;
LOG("replaceWord3 %s", word3_);
	char word[100];
	strcpy(word, word3_);
     e_->cut(); 
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord4() {
	if (*word4_ == ' ' || *word4_ == '\0')
		return;
LOG("replaceWord4 %s", word4_);
	char word[100];
	strcpy(word, word4_);
     e_->cut(); 
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord5() {
	if (*word5_ == ' ' || *word5_ == '\0')
		return;
LOG("replaceWord5 %s", word5_);
	char word[100];
	strcpy(word, word5_);
     e_->cut(); 
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::spellIgnore()
{
	QString text = e_->markedText();
	QCString utext = text.utf8();
	const char *ctext = utext; 
	char *active_word = extract_word((char *) ctext);
	if (active_word == 0) {
		active_word = extract_word((char *) (ctext + 1));
		if (active_word == 0) {
			spellAdvance();
			return;
		}
	}
	LOG("ignore %s", active_word);	
     pdict_->getIgnore()->setString(active_word);
	delete[] active_word;

	spellAdvance();
}

void Editor::spellAdd()
{
	QString text = e_->markedText();
	QCString utext = text.utf8();
	const char *ctext = utext; 
	char *active_word = extract_word((char *) ctext);
	if (active_word == 0) {
		active_word = extract_word((char *) (ctext + 1));
		if (active_word == 0) {
			spellAdvance();
			return;
		}
	}
	LOG("add %s", active_word);	
	pdict_->getUser()->setString(active_word);
	delete[] active_word;
	
	spellAdvance();
}

void Editor::spellName()
{
	QString text = e_->markedText();
	QCString utext = text.utf8();
	const char *ctext = utext; 
	char *active_word = extract_word((char *) ctext);
	if (active_word == 0) {
		active_word = extract_word((char *) (ctext + 1));
		if (active_word == 0) {
			spellAdvance();
			return;
		}
	}
	LOG("name %s", active_word);	
     pdict_->getFixed()->setString(active_word, true);
	delete[] active_word;
	
	spellAdvance();
}


void Editor::textChanged()
{
//LOG("Text changed");
    changed_ = TRUE;
//lucian: is coming here for each key press;
    cleanMWords();
}
void Editor::clicked(int para, int pos)
{
LOG("clicked %d/%d", para, pos);
//lucian: is coming here for each key press;
    cleanMWords();
}

void Editor::undo() {
    cleanMWords();
    e_->undo(); 
}

void Editor::cut() {
LOG("Editor cut");
    cleanMWords();
    e_->cut(); 
}
void Editor::copy() {
    cleanMWords();
    e_->copy(); 
}
void Editor::paste() {
    cleanMWords();
    e_->paste(); 
}
void Editor::e_delete() {
    cleanMWords();
    e_->del(); 
}

void Editor::search() {
    cleanMWords();
	SearchDialog *sd = new SearchDialog();
	sd->setCaption("Enter search word");
	sd->exec();
	QString sword = sd->getWord();
	search_ = sword;
	
	if (sword == "")
		return;
	QCString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::search %s", cstr);
	delete sd;
	
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	int lines = e_->numLines();
	
	while (para < lines) {
		QString line = e_->textLine(para);
		index = line.find(sword, index);
		if (index != -1) {
			e_->setCursorPosition(para, index);
			return;
		}
		index = 0;
		para++;
	}
}

void Editor::searchAgain() {
    cleanMWords();
	if (search_ == "")
		return;
	QCString utext = search_.utf8();
	const char *cstr = utext;
	LOG("Editor::searchAgain %s", cstr);
	
	e_->moveCursor(QTextEdit::MoveForward, false);
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	int lines = e_->numLines();
	
	while (para < lines) {
		QString line = e_->textLine(para);
		index = line.find(search_, index);
		if (index != -1) {
			e_->setCursorPosition(para, index);
			return;
		}
		index = 0;
		para++;
	}
	e_->moveCursor(QTextEdit::MoveBackward, false);
}

void Editor::gotoLine() {
    cleanMWords();
	LineDialog *ld = new LineDialog();
	ld->setCaption("Enter line number");
	ld->exec();
	int line = ld->getLine();
	LOG("Editor::gotoline %d", line);
	delete ld;
	
	if (line > 0) {
		int lines = e_->numLines();
	     if (line > lines)
			line = lines;
		e_->setCursorPosition(line - 1, 0);
	}
}

void Editor::gotoBookmark() {
	cleanMWords();
	BookmarkDialog *bd = new BookmarkDialog();
	bd->setCaption("Enter bookmark number");
	bd->exec();
	int bookmark = bd->getBookmark();
	LOG("Editor::gotoBookmark %d", bookmark);
	delete bd;
			
	ASSERT(bookmark < MAXBOOKMARKS);
	if (bookmark > MAXBOOKMARKS)
		return;
	int line = ybook_[bookmark];
	int lines = e_->numLines();
     if (line > lines)
		line = lines - 1;
	e_->setCursorPosition(line, xbook_[bookmark]);
}

void Editor::setBookmark() {
	cleanMWords();
	BookmarkDialog *bd = new BookmarkDialog();
	bd->setCaption("Enter bookmark number");
	bd->exec();
	int bookmark = bd->getBookmark();
	LOG("Editor::setBookmark %d", bookmark);
	delete bd;
	
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	ASSERT(bookmark < MAXBOOKMARKS);
	if (bookmark > MAXBOOKMARKS)
		return;
	xbook_[bookmark] = index;
	ybook_[bookmark] = para;
}


void Editor::unicode_a() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("ă");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}

void Editor::unicode_A() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("Ă");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}

void Editor::unicode_s() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("ş");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}

void Editor::unicode_S() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("Ş");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}

void Editor::unicode_q() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("â");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}

void Editor::unicode_Q() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("Â");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}

void Editor::unicode_t() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("ţ");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}

void Editor::unicode_T() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("Ţ");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}

void Editor::unicode_i() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("î");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}

void Editor::unicode_I() {
    cleanMWords();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8("Î");
	e_->insertAt(str, para, index, false);
	e_->setCursorPosition(para, index + 1);
}



LineDialog::LineDialog(): QDialog(0, "Enter line number", true) {
	QVBoxLayout *vly = new QVBoxLayout(this);
	vly->setAutoAdd(true);
	
	QHBox *hbox = new QHBox(this);
	label_ = new QLabel("Line: ", hbox);
	text_ = new QLineEdit(hbox);
	button_ = new QPushButton("Ok", hbox);
	
	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}

int LineDialog::getLine() {
	int retval;
	
	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		bool ok;
		retval = str.toInt(&ok);
		if (!ok || retval <= 0) {
			retval = -1;
			QMessageBox::warning(0,"Error", "Not a valid line number");
		}
	}
	return retval;
}

BookmarkDialog::BookmarkDialog(): QDialog(0, "Enter bookmark number", true) {
	QVBoxLayout *vly = new QVBoxLayout(this);
	vly->setAutoAdd(true);
	
	QHBox *hbox = new QHBox(this);
	label_ = new QLabel("Bookmark: ", hbox);
	text_ = new QLineEdit(hbox);
	button_ = new QPushButton("Ok", hbox);
	
	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}

int BookmarkDialog::getBookmark() {
	int retval;
	
	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		bool ok;
		retval = str.toInt(&ok);
		if (!ok || retval < 0 || retval >= Editor::MAXBOOKMARKS) {
			retval = -1;
			QMessageBox::warning(0,"Error", "Not a valid bookmark number");
		}
	}
	return retval;
}

SearchDialog::SearchDialog(): QDialog(0, "Enter search word", true) {
	QVBoxLayout *vly = new QVBoxLayout(this);
	vly->setAutoAdd(true);
	
	QHBox *hbox = new QHBox(this);
	label_ = new QLabel("Word: ", hbox);
	text_ = new QLineEdit(hbox);
	button_ = new QPushButton("Ok", hbox);
	
	connect(button_, SIGNAL(clicked()), SLOT(accept()));
	QAccel *a = new QAccel(this);        // create accels for myWindow
	a->connectItem(a->insertItem(ALT+Key_A), this, SLOT(unicode_a()));
	a->connectItem(a->insertItem(SHIFT+ALT+Key_A), this, SLOT(unicode_A()));
	a->connectItem(a->insertItem(ALT+Key_S), this, SLOT(unicode_s()));
	a->connectItem(a->insertItem(SHIFT+ALT+Key_S), this, SLOT(unicode_S()));
	a->connectItem(a->insertItem(ALT+Key_Q), this, SLOT(unicode_q()));
	a->connectItem(a->insertItem(SHIFT+ALT+Key_Q), this, SLOT(unicode_Q()));
	a->connectItem(a->insertItem(ALT+Key_T), this, SLOT(unicode_t()));
	a->connectItem(a->insertItem(SHIFT+ALT+Key_T), this, SLOT(unicode_T()));
	a->connectItem(a->insertItem(ALT+Key_I), this, SLOT(unicode_i()));
	a->connectItem(a->insertItem(SHIFT+ALT+Key_I), this, SLOT(unicode_I()));
}

QString SearchDialog::getWord() {
	QString retval;
	
	if (text_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text_->text();
	}
	return retval;
}

void SearchDialog::unicode_a() {
	QString str = QString::fromUtf8("ă");
	text_->insert(str);
	text_->cursorForward(false);
}

void SearchDialog::unicode_A() {
	QString str = QString::fromUtf8("Ă");
	text_->insert(str);
	text_->cursorForward(false);
}

void SearchDialog::unicode_s() {
	QString str = QString::fromUtf8("ş");
	text_->insert(str);
	text_->cursorForward(false);
}

void SearchDialog::unicode_S() {
	QString str = QString::fromUtf8("Ş");
	text_->insert(str);
	text_->cursorForward(false);
}

void SearchDialog::unicode_q() {
	QString str = QString::fromUtf8("â");
	text_->insert(str);
	text_->cursorForward(false);
}

void SearchDialog::unicode_Q() {
	QString str = QString::fromUtf8("Â");
	text_->insert(str);
	text_->cursorForward(false);
}

void SearchDialog::unicode_t() {
	QString str = QString::fromUtf8("ţ");
	text_->insert(str);
	text_->cursorForward(false);
}

void SearchDialog::unicode_T() {
	QString str = QString::fromUtf8("Ţ");
	text_->insert(str);
	text_->cursorForward(false);
}

void SearchDialog::unicode_i() {
	QString str = QString::fromUtf8("î");
	text_->insert(str);
	text_->cursorForward(false);
}

void SearchDialog::unicode_I() {
	QString str = QString::fromUtf8("Î");
	text_->insert(str);
	text_->cursorForward(false);
}




WwwDialog::WwwDialog(): QDialog(0, "Enter www location", true) {
	QVBoxLayout *vly = new QVBoxLayout(this);
	vly->setAutoAdd(true);
	
	QHBox *hbox = new QHBox(this);
	label_ = new QLabel("url: ", hbox);
	text_ = new QLineEdit(hbox);
	button_ = new QPushButton("Ok", hbox);
	
	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}

QString WwwDialog::getWord() {
	QString retval;
	
	if (text_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text_->text();
	}
	return retval;
}

StatusDialog::StatusDialog(QFont &fnt, char *msg): QDialog(0, "File status", true) {
	ASSERT(msg != 0);
	QVBoxLayout *vly = new QVBoxLayout(this);
	vly->setAutoAdd(true);
	
	QHBox *hbox = new QHBox(this);
	
	QString qstr = QString::fromUtf8(msg);
				
	label_ = new QLabel(qstr, hbox);
	label_->setFont(fnt);
	button_ = new QPushButton("Ok", hbox);
	
	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}


